"""
This module provides functions to report weather reports for cities.

Bla bla bla
Bla bla bla
Bla bla bla
Bla bla bla
"""

__version__ = "1.0.7"

__author__ = "Jürgen Brauer"

DEFAULT_FIGURE_TITLE = "Temperature over time"
NOMINATIM_USER_AGENT = "gjkgjkfdlnv"


from datetime import timedelta, datetime

import matplotlib.pyplot as plt
import pandas
from geopy.geocoders import Nominatim
from meteostat import Point, Daily




def generate_temperature_plot(df : pandas.DataFrame,
                              filename : str = "temperature_plot.pdf",
                              windowsize : int = 1,
                              figtitle : str = DEFAULT_FIGURE_TITLE):
    """
    Given a Pandas DataFrame with temperature data, generate a plot and save it as a file.

    :param df: The dataframe from which to retrieve the temperature data.
    :param filename: Name of the plot file
    :param windowsize: Number of time steps used for the sliding window.
    :param figtitle: Title of the figure.
    :return: Nothing.
    """
    
    if not isinstance(df, pandas.DataFrame):
        print("Bitte füttere mich nur mit DataFrames!")
        return

    if not isinstance(filename, str):
        print("Bitte gibt als Dateinamen nur Zeichenketten an!")
        return
    
    y1 = df["tmax"].rolling(windowsize).median()
    y2 = df["tmin"].rolling(windowsize).median()
    y3 = df["tavg"].rolling(windowsize).median()
    
    plt.figure( figsize=(10,4) )
    plt.plot( y1, color="darkred", label="Maximaltemperatur [°C]"  )
    plt.plot( y2, color="blue", label="Minimaltemperatur [°C]" )
    plt.plot( y3, color="black", label="Durchschnittstemperatur [°C]", linestyle="--" )
    plt.title( figtitle )
    plt.xlabel("Datum", fontsize=16)
    plt.ylabel("Temperatur [°C]", fontsize=16)
    plt.xticks( rotation=45 )
    plt.fill_between(df.index, y1,y2, color="lightgray")    
    plt.annotate(xy=(y2.idxmin(), y2.min() ),
                 xytext=(y2.idxmin() + timedelta(3*30), y2.min() ),
                 text=f"Hier war es kalt! ({y2.min()} °C)",
                 color="red",
                 arrowprops = {"arrowstyle" : "->",
                               "lw" : 1.5,
                               "color" : "red"}
                 )
    plt.axvline(y2.idxmin(), linestyle="--", lw=4, color="red")
    plt.legend(loc="upper left")
    plt.tight_layout()
    plt.savefig( filename )


def get_coordinates_for_address(address : str) -> tuple:
    """
    For a given address (e.g., a city name) return the (latitude,longitude) coordinate
    """
    geolocator = Nominatim(user_agent=NOMINATIM_USER_AGENT)
    location = geolocator.geocode(address)
    return location.latitude, location.longitude


def get_weather_for_address(address : str,
                            startdatetime : datetime,
                            enddatetime: datetime) -> pandas.DataFrame:
    """
    For a given address (e.g., a city name) and time interval return a table with weather data

    Bla bla bla
    """
    lat, long = get_coordinates_for_address(address)
    point = Point(lat, long)
    data = Daily(point, startdatetime, enddatetime)
    df = data.fetch()
    return df


def generate_weather_plots_for_cities(list_cities : list[str],
                                      startdatetime : datetime,
                                      enddatetime: datetime) -> None:
    """
    For a given list of city names, retrieve weather data and generate temperature plots.

    Bla bla bla
    """
    
    for city in list_cities:
        df = get_weather_for_address(city, startdatetime, enddatetime)
        generate_temperature_plot(df,
                                  figtitle=f"Temperature plot for {city}",
                                  filename=f"weather_for_{city}.pdf")
